/*
 *  PlaySound - A MatLab MEX file to play a soundfile
 *  'after an original idea by Malcolm Slaney..'
 *  .. this one modified to use dpwe's hardware code
 *  dpwe 1994feb10
 *
 */
 
#include    "dpwelib.h"		/* instead of stdio.h */
#include    <math.h>
#include    <ctype.h>
#include    "audIO.h"
#include 	"sndfutil.h"	/* for conver-to-double function */
#include    "mex.h"

#define     MAX_TEXT    300     /* Maximum size of string option */

#ifdef  THINK_C             /* This is needed with ThinkC 5.0 */
/* short __D_INF[2]; */
short __X_INF[2];
void abort(void)
{
	Debugger();
}
FILE *errfile = NULL;
#define RETCHAR "\r"
#else
#define RETCHAR "\n"
#endif

void mexret(void) 
{   /* special fn to print return since it's such a portable pain */
	mex_printf(RETCHAR);
}

void MyDouble2Short(register double *src, register short *dst, register long n)
{	/* do the format conversion */
	register double k = 32767.0;
#ifndef THINK_C
	
	while(n--)
		*dst++ = k * *src++;
#else
	asm {
;		movea.l	8(A6), A0
	move.l	src,A0
;		movea.l 12(A6), A1
	move.l  dst,A1
		fmove.d #32767.0, FP1
;		move.l  16(A6), D1
	move.l	n,D1
		beq.s	@4			; skip out if D1 is zero
@3
		fmove.d (A0)+, FP0
		fmul	FP1, FP0
		fmove.w	FP0, (A1)+
		subq.l	#1, D1
		bne.s	@3
@4
		}
#endif /* THINK_C */
}

/* have to use non-ANSI prototype for cc on DECstations 
   (it's a long story - believe me) */


#ifdef __STDC__
void
mexFunction(int nlhs, Matrix *plhs[], int nrhs, Matrix *prhs[])
#else /* !__STDC__ - proper C compiler, prototypes */
/* have to use non-ANSI prototype for cc on DECstations 
   (it's a long story - beleive me) */
mexFunction(nlhs, plhs, nrhs, prhs)
  int nlhs;
  Matrix *plhs[];
  int nrhs;
  Matrix *prhs[];
#endif /* __STDC__ */
{
  float   sampleRate;
  long    Length, channels;
  int     wordSize;
  int 	i, err;
  AUSTRUCT *aus;
  long   	pvl, pvb[16];
  short 	*sbuf = NULL;
  long 	sbsize = 32768;		/* frames written at once */

#ifdef DEBUG
  mex_printf("PlaySound: Got %d lhs args and %d rhs args.", 
	     nlhs, nrhs);	mexret();
  for (i=0;i<nrhs;i++)  {
    mex_printf("Arg #%d is size %d x %d", 
	       (long)i, mxGetM(prhs[i]), mxGetN(prhs[i]));	mexret();
  }
  for (i=0;i<nlhs;i++)
    if (plhs[i])  {
      mex_printf("Arg #%d is size %d x %d", 
		 (long)i, mxGetM(plhs[i]), mxGetN(plhs[i]));
	  mexret();
	}
#endif				/* DEBUG */

  if (nrhs < 2){
    mex_printf(
	       "PlaySound  PlaySound(D,SR[,B])  plays a sound on local hardware."); mexret();
    mex_printf(
	       "           The data D should be scaled between -1.0 and 1.0,"); mexret();
    mex_printf(
	       "           with one row for each channel.  SR specifies the"); mexret();
    mex_printf(
	       "           sampling rate.  B, if present, specifies the buffer"); mexret();
    mex_printf(
	       "           size i.e. how many frames to convert at once (default:%ld).", sbsize); mexret();
    return;
  }
  /* look at the data */
  Length   = mxGetN(prhs[0]);
  channels = mxGetM(prhs[0]);
  if(channels > 16)  {		/* probably transposed - best not to write it */
    mexErrMsgTxt("There must be 16 or less channels (rows).");
    return;
  }
  sampleRate = *(mxGetPr(prhs[1]));
  if(nrhs >= 3)  {		/* buffer size was specified */
	double specbuf = *(mxGetPr(prhs[2]));
  	if(specbuf > 1000)	/* ignore if zero or negative */
  		sbsize = specbuf;
  }

#ifdef DEBUG
  mex_printf("PlaySound: %d chans, %ld frames, SR = %f", 
	     channels, Length, sampleRate); mexret();
#endif				/* DEBUG */
  /* Open the audio port */
  if((aus=AUOpen(SFMT_SHORT, sampleRate, channels, sbsize/2, AIO_OUTPUT)) == NULL){
    mexErrMsgTxt("Problem initializing sound hardware.");
    return;
  }

  if( (sbuf = malloc(sbsize*channels*sizeof(short)))== NULL)  {
    AUClose(aus,1);
    mexErrMsgTxt("Could not allocate buffer for playing.");
    return;
  }
  {
    /* convert the doubles into buffer, play them out */
    long  todo, thistime;
    double *dsrc = mxGetPr(prhs[0]);

    todo = Length;
    while(todo)  {
      thistime = (todo > sbsize)?sbsize:todo;

 /*
      ConvertSampleBlockFormat((char *)dsrc, (char *)sbuf, 
			       SFMT_DOUBLE, SFMT_SHORT, 
			       thistime*channels);
  */
	  MyDouble2Short(dsrc, sbuf, thistime*channels);
      AUWrite(aus, sbuf, thistime);
      todo -= thistime;
      dsrc += thistime*channels;
    }
#ifdef DEBUG
    mex_printf("PlaySound: data written"); mexret();
#endif				/* DEBUG */
  } 
  if(sbuf)
    free(sbuf);
#ifdef THINK_C
{
   int blks, under;
	AUGetStats(aus, &blks, &under);
	mex_printf("completed %d blocks, %d underruns", blks, under); mexret();
}
#endif /* THINK_C */
  AUClose(aus,0);
#ifdef DEBUG
  mex_printf("PlaySound: returning..."); mexret();
#endif				/* DEBUG */
}

