/*
 *  ReadSound - A MatLab MEX file to read an AIFF/C soundfile
 *  'after an original idea by Malcolm Slaney..'
 *  .. this one modified to use dpwe's AIF reading code etc.
 *  dpwe 1994feb10
 *
 */
 
#include    "dpwelib.h"		/* instead of stdio.h */
#include    <math.h>
#include    <ctype.h>
#include 	"aifif.h"
#include 	"sndfutil.h"	/* for conver-to-double function */
#include    "mex.h"

#define     MAX_TEXT    300     /* Maximum size of string option */

/* #define DEBUG */

#ifdef  THINK_C             /* This is needed with ThinkC 5.0 */
/* short __D_INF[2]; */
short __X_INF[2];
void abort(void)
{
	Debugger();
}
FILE *errfile = NULL;
#define RETCHAR "\r"
#else
#define RETCHAR "\n"
#endif

void mexret() 
{   /* special fn to print return since it's such a portable pain */
	mex_printf(RETCHAR);
}

/* has to be outside to initialize right */
static    AiffID noneId = NONE_ID;


#ifdef __STDC__
void
mexFunction(int nlhs, Matrix *plhs[], int nrhs, Matrix *prhs[])
#else /* !__STDC__ - proper C compiler, prototypes */
/* have to use non-ANSI prototype for cc on DECstations 
   (it's a long story - beleive me) */
mexFunction(nlhs, plhs, nrhs, prhs)
  int nlhs;
  Matrix *plhs[];
  int nrhs;
  Matrix *prhs[];
#endif /* __STDC__ */
{
    float   sampleRate;
    char    fileName[MAX_TEXT];
    long    Length, channels;
    long    base, len;
    int     wordSize;
    int 	i, err;
    AIF_STRUCT *aifs;
    long   	pvl, pvb[16];

#ifdef DEBUG
    mex_printf("ReadSound: Got %d lhs args and %d rhs args.", 
			   nlhs, nrhs); mexret();
    for (i=0;i<nrhs;i++) {
        mex_printf("Arg #%d is size %d x %d", 
		   (long)i, mxGetM(prhs[i]), mxGetN(prhs[i]));
		mexret();
	}
    for (i=0;i<nlhs;i++)
        if (plhs[i]) {
            mex_printf("Arg #%d is size %d x %d", 
		       (long)i, mxGetM(plhs[i]), mxGetN(plhs[i]));
		    mexret();
		}
#endif /* DEBUG */

    if (nrhs < 1){
        mex_printf(
		 "ReadSound  [D,SR,L,C] = ReadSound(FILE[,LEN[,BASE]])  reads an AIFF/C ");mexret();
	mex_printf(
		 "           soundfile specified by filename FILE, and returns the sound");mexret();
	mex_printf(
		 "           data scaled between -1.0 and 1.0, one row for each");mexret();
	mex_printf(
		 "           channel.  LEN and BASE describe how much sound to");mexret();
	mex_printf(
		 "           return and where to read it from (BASE defaults to 0,");mexret();
	mex_printf(
		 "           LEN to whole file.)");mexret();
mex_printf(
		 "           If SR is provided, it is set to the file's sampling rate. ");mexret();
    mex_printf(
		 "           L gets the total length and C the channel count.");mexret();
		return;
    }
	if (!mxIsString(prhs[0])){
		mexErrMsgTxt("Argument must be a string (filename)");
		return;
    } else {
        int j;
        double *pd;

		pd = mxGetPr(prhs[0]);
		/* Convert to String */
		for (j=0;j<mxGetN(prhs[0]) && j < MAX_TEXT;j++)  {
            fileName[j] = /* tolower */ ((char)*pd); /* Unix case sensitive */
			++pd;
		}
        fileName[j] = 0;
    }
    if(nrhs>1)  {
	len = *mxGetPr(prhs[1]);
    } else {
	len = -1;
    }
    if(nrhs>2)  {
	base = *mxGetPr(prhs[2]);
    } else {
	base = 0;
    }

#ifdef DEBUG
    mex_printf("ReadSound: filename is '%s', len %ld, base %ld", 
	       fileName,len,base);mexret();
#endif /* DEBUG */
	/* Open the soundfile */
    aifs = aifNew();
    if( (err = aifOpenRead(aifs, fileName))==AIF_E_NO_ERR )  {
		pvl = 0;
		pvb[pvl] = AIF_P_SAMPRATE;	pvl+=2;
		pvb[pvl] = AIF_P_CHANNELS;	pvl+=2;
		pvb[pvl] = AIF_P_NFRAMES;	pvl+=2;
		pvb[pvl] = AIF_P_SAMPSIZE;	pvl+=2;
		pvb[pvl] = AIF_P_FILETYPE;	pvl+=2;
		pvb[pvl] = AIF_P_COMPID;	pvl+=2;
		aifGetParams(aifs, pvb, pvl);
		sampleRate = FLOATofLONG(pvb[1]);
		channels   = pvb[3];
		Length     = pvb[5];
		wordSize   = (pvb[7]+BITS_PER_BYTE-1)/BITS_PER_BYTE;
		if(pvb[9] == AIF_FT_AIFC && !aifIdCmp((AiffIDPtr)&pvb[11], 
										  (AiffIDPtr) ADR noneId))  {
			aifClose(aifs);
			aifFree(aifs);
			mexErrMsgTxt("Cannot read compressed AIFC data.");
			return;
		}
		/* we should be OK now */
		if(len <= 0)		len = Length;
		if(base+len > Length)	len = Length - base;
		if(base > Length)	{ base = 0; len = 0; }
#ifdef DEBUG 
		mex_printf("ReadSound: file is %ld frames of %d chans, %d bps, SR %f; read %ld+%ld",
				   Length, channels, wordSize, sampleRate,base,len);mexret();
#endif /* DEBUG */
		if (nlhs > 0){
			Matrix  *soundMatrix;
			double *pd;
			int infmt;

			soundMatrix = mxCreateFull(channels, len, REAL);
#ifdef DEBUG
			mex_printf("ReadSound: soundMatrix created");mexret();
#endif /* DEBUG */
			plhs[0] = soundMatrix;
			pd = mxGetPr(soundMatrix);
			/* read *whole file* */
			aifFrameSeek(aifs, base);
			aifReadFrames(aifs, pd, len);
#ifdef DEBUG
			mex_printf("ReadSound: frames read");mexret();
#endif /* DEBUG */
			/* .. then convert to doubles */
			if(wordSize == 1) 
				infmt = SFMT_CHAR;
			else if(wordSize == 4)
				infmt = SFMT_LONG;
			else 
				infmt = SFMT_SHORT;	/* please.. no 24 bit files! */
			ConvertSampleBlockFormat((char *)pd, (char *)pd, 
									 infmt, SFMT_DOUBLE, 
									 channels * len);
#ifdef DEBUG
			mex_printf("ReadSound: buffer converted");mexret();
#endif /* DEBUG */
        }
		if (nlhs > 1){
			Matrix *srMatrix;
			double *pd;
	    
			srMatrix = mxCreateFull(1, 1, REAL);
			plhs[1] = srMatrix;
			pd = mxGetPr(srMatrix);
			*pd = sampleRate;
#ifdef DEBUG
			mex_printf("ReadSound: sampleRate created");mexret();
#endif /* DEBUG */
		}
		if (nlhs > 2){
			Matrix *matrix;
			double *pd;
	    
			matrix = mxCreateFull(1, 1, REAL);
			plhs[2] = matrix;
			pd = mxGetPr(matrix);
			*pd = Length;
		}
		if (nlhs > 3){
			Matrix *matrix;
			double *pd;
	    
			matrix = mxCreateFull(1, 1, REAL);
			plhs[3] = matrix;
			pd = mxGetPr(matrix);
			*pd = channels;
		}
		aifClose(aifs);
    } else {
#ifdef THINK_C
   if(errfile)
   	   fclose(errfile);
#endif /* THINK_C */
		mex_printf("aif error: %s", aifErrMsg(err));mexret();
		mexErrMsgTxt("Could not open file.");
	}
	aifFree(aifs);
#ifdef THINK_C
   if(errfile)
   	   fclose(errfile);
#endif /* THINK_C */
#ifdef DEBUG
	mex_printf("ReadSound: returning...");mexret();
#endif /* DEBUG */
}

