/*
 *  WriteSound - A MatLab MEX file to write a soundfile
 *  'after an original idea by Malcolm Slaney..'
 *  .. this one modified to use dpwe's AIFF/C code etc.
 *  dpwe 1994feb10
 *
 */
 
#include    "dpwelib.h"		/* instead of stdio.h */
#include    <math.h>
#include    <ctype.h>
#include 	"aifif.h"
#include 	"sndfutil.h"	/* for conver-to-double function */
#include    "mex.h"

#define     MAX_TEXT    300     /* Maximum size of string option */

/* has to be outside to initialize right */
static    AiffID noneId = NONE_ID;

#ifdef  THINK_C             /* This is needed with ThinkC 5.0 */
/* short __D_INF[2]; */
short __X_INF[2];
void abort(void)
{
	Debugger();
}
FILE *errfile = NULL;
#define RETCHAR "\r"
#else
#define RETCHAR "\n"
#endif

void mexret(void) 
{   /* special fn to print return since it's such a portable pain */
	mex_printf(RETCHAR);
}

/* have to use non-ANSI prototype for cc on DECstations 
   (it's a long story - beleive me) */

#ifdef __STDC__
void
mexFunction(int nlhs, Matrix *plhs[], int nrhs, Matrix *prhs[])
#else /* !__STDC__ - proper C compiler, prototypes */
/* have to use non-ANSI prototype for cc on DECstations 
   (it's a long story - beleive me) */
mexFunction(nlhs, plhs, nrhs, prhs)
  int nlhs;
  Matrix *plhs[];
  int nrhs;
  Matrix *prhs[];
#endif /* __STDC__ */
{
    float   sampleRate;
    char    fileName[MAX_TEXT];
    long    Length, channels;
    int     wordSize;
    int 	i, err;
    AIF_STRUCT *aifs;
    long   	pvl, pvb[16];
	short 	*sbuf = NULL;
	long 	sbsize = 32768;	/* frames written at once */

#ifdef DEBUG
    mex_printf("WriteSound: Got %d lhs args and %d rhs args.", 
			   nlhs, nrhs);mexret();
    for (i=0;i<nrhs;i++)  {
        mex_printf("Arg #%d is size %d x %d", 
		   (long)i, mxGetM(prhs[i]), mxGetN(prhs[i]));
		mexret();
	}
    for (i=0;i<nlhs;i++)
        if (plhs[i])  {
            mex_printf("Arg #%d is size %d x %d", 
		       (long)i, mxGetM(plhs[i]), mxGetN(plhs[i]));
		    mexret();
		}
#endif /* DEBUG */

    if (nrhs < 3){
        mex_printf(
		 "WriteSound  WriteSound(FILE,D,SR)  writes an AIFC soundfile");mexret();
		mex_printf(
		 "           to filename FILE containing the data in D which ");mexret();
		mex_printf(
		 "           should be scaled between -1.0 and 1.0, with one row");mexret();
		mex_printf(
		 "           for each channel.  SR specifies the file's");mexret();
		mex_printf(
		 "           sampling rate.");mexret();
		return;
    }
	if (!mxIsString(prhs[0])){
		mexErrMsgTxt("Argument 1 must be a string (filename).");
		return;
    } else {
        int j;
        double *pd;

		pd = mxGetPr(prhs[0]);
		/* Convert to String */
		for (j=0;j<mxGetN(prhs[0]) && j < MAX_TEXT;j++)  {
            fileName[j] = /* tolower */((char)*pd);  /* mixed case unix */
			++pd;
		}
        fileName[j] = 0;
    }
	/* look at the data */
	Length   = mxGetN(prhs[1]);
	channels = mxGetM(prhs[1]);
	if(channels > 16)  {	/* probably transposed - best not to write it */
		mexErrMsgTxt("There must be 16 or less channels (rows).");
		return;
	}
	sampleRate = *(mxGetPr(prhs[2]));
#ifdef DEBUG
    mex_printf("WriteSound: filename '%s', %ld chans, %ld frames, SR = %f", 
			   fileName, channels, Length, sampleRate);mexret();
#endif /* DEBUG */
	/* Open the soundfile */
    aifs = aifNew();
	/* set up the params */
	pvl = 0;
	pvb[pvl++] = AIF_P_SAMPRATE;	pvb[pvl++] = LONGofFLOAT(sampleRate);
	pvb[pvl++] = AIF_P_CHANNELS;	pvb[pvl++] = channels;
	pvb[pvl++] = AIF_P_NFRAMES;		pvb[pvl++] = Length;
	pvb[pvl++] = AIF_P_SAMPSIZE;	pvb[pvl++] = 16;	/* bits per sample */
    pvb[pvl++] = AIF_P_FILETYPE;    pvb[pvl++] = AIF_FT_AIFF;       /* force AIFF output */
 	aifSetParams(aifs, pvb, pvl);

	sbsize = 10000;	/* write in chunks this size */
	if( (sbuf = malloc(sbsize*channels*sizeof(short)))== NULL)  {
		aifFree(aifs);
		mexErrMsgTxt("Could not allocate buffer for writing.");
		return;
	}
    if( (err = aifOpenWrite(aifs, fileName, 2*channels*Length))==AIF_E_NO_ERR )  {
		/* allocate a buffer, convert the doubles into it, write them out */
		long  todo, thistime;
		double *dsrc = mxGetPr(prhs[1]);

		todo = Length;
		while(todo)  {
			thistime = (todo > sbsize)?sbsize:todo;
			ConvertSampleBlockFormat((char *)dsrc, (char *)sbuf, 
									 SFMT_DOUBLE, SFMT_SHORT, 
									 thistime*channels);
			aifWriteFrames(aifs, sbuf, thistime);
			todo -= thistime;
			dsrc += thistime*channels;
		}
#ifdef DEBUG
		mex_printf("WriteSound: data written");mexret();
#endif /* DEBUG */
		aifClose(aifs);
    } else {
		mex_printf("aif error: %s", aifErrMsg(err));mexret();
		mexErrMsgTxt("Could not open file.");
	}
	aifFree(aifs);
	if(sbuf)
		free(sbuf);
#ifdef DEBUG
	mex_printf("WriteSound: returning...");mexret();
#endif /* DEBUG */
}

#ifdef  THINK_C             /* This is needed with ThinkC 5.0 */
/* short __D_INF[2]; */
short __X_INF[2];
#endif
