function [B,A,T,BW] = bilbpf(SR,CF,Q,N)
% [B,A,T,BW] = bilbpf(SR,CF,Q,N) Design an IIR bpass filter using bilinear t/f.
%	As an alternative to iirbpfilt.m, this designs some proper bandpass 
%	filters without alias problems.  Based on Neal's function.  
%	B,A are filter coeffs, SR is sample rate, CF is centre frequency 
%	in same units, Q is the Q factor, and N is not actually used at the 
%	moment (was to be filter order).
%	T is a vector of group delays in samples for each filter.
%	BW is the bandwidths in Hz.
% dpwe 1994jul12.

designQ = Q/N;
w0 = 2*pi*CF;
nums = [ 0 1/(w0*designQ) 0];
dens = [ 1/(w0^2) 1/(w0*designQ) 1];
[b,a] = bilinear(nums, dens, SR, CF);
convnums = 1;
convdens = 1;
for n = 1:N
  convnums = conv(convnums, b);
  convdens = conv(convdens, a);
end
B = convnums;
A = convdens;
% T = talign(B,A,N);
T = SR*(N-1)./((4*pi/Q)*CF);
BW = CF/Q;
