% ~/matlab/gtones-1997feb/demo_gt.m
% Demonstrate using the gammatone functions

% Load a soundfile using the SoundMex4.1 extensions

soundfile='brn1h.aif';
[da,sr] = ReadSound(soundfile);
% da is the samples, sr is the sample rate
% Check we can play it
PlaySound(da, sr);

% Design the filterbank - 6 bands per octave from 100 Hz up
fmin = 100;
bandsperoct = 6;
% How ever many bands will fit under the nyquist limit
bands = floor(bandsperoct * log((sr/2)/fmin)/log(2));

% Q-factor is 8, number of pole-pairs is 2
Q = 8;
N = 2;

% bpfiltbank designs several different types of filterbank.
% Type 2 is the Lyon-Slaney gammatone
[B,A,T,W,F] = bpfiltbank(sr, fmin, bandsperoct, bands, Q, N, 2);
% Each row of B and A define one filter (zero and pole coeffs)
% T is a vector of phase-lags (delays); W is the bandwidths; F the ctr freqs.


% Run the filterbank over the signal.  Don't subsample or rectify the 
% signal, but do compensate for the delays returned by the design 
% function.
df = filterbank(B, A, da, 0, 0, T);

% Each row is a full-sampling-rate bandpass signal
PlaySound(df(10,:), sr);

% Rectify & subsample by 100 for display
subfact = 100;
ds = sqrt(subsmooth(df.^2, subfact));
imgdb(ds)
gcolor

% Recombine & compare with original
dr = ifilterbank(B,A,df,1);
PlaySound(da, sr);
PlaySound(dr, sr);

% .. or remodulate the subsampled surface onto noise
% (this involves a very slow, but one-off, interband overlap calculation)
du = invsurf(ds, sr, sr/subfact, B, A, T, W);
PlaySound(da, sr);
PlaySound(du, sr);
