function Y = fracdelay(X, D, O)
% Y = fracdelay(X, D, O)  Fractional delay
%	Y is the same length as X, but subjected to a delay of D samples, 
%	where D can be non-integer.  Accomplished with an FIR filter 
%	of order O (default 6).
% dpwe@media.mit.edu 1996jan16

if(nargin<3)
  O = 6;
end

nr = size(X,1);
sx = size(X,2);

Y = zeros(nr,sx);


for(r = 1:nr)
  % Construct a filter with a delay of d2+FD
  if(r==1 | vsize(D)>1)
    % Separate integer and fractional parts
    ID = floor(D(r));
    FD = D(r) - ID;

    d2 = floor((O-1)/2);

    zz = zeros(1,max(0,-D(r)));

    % Construct a filter with a delay of d2+FD
    fi = sinc(-FD+(-(d2)+[0:(O-1)]));
  end

  yy = conv(fi, [X(r,:),zz]);

  if(d2 > ID)
    Y(r,:) = yy((1+d2-ID)+[0:(sx-1)]);
  else % (d2 <= ID)
    Y(r,:) = [zeros(1,ID-d2),yy(1+[0:(sx-1-(ID-d2))])];
  end
end
