function G = gammair(T, F, B, N)
% G = gammair(T, F, B, N)                 Patterson gammatone impulse response.
%	G is a set of samples of the impulse response at times in T 
%	for the gammatone filter centered at frequency F with bandwidth B, 
%	and order N (defaults to 4), i.e.
%	G(T) = T^(N-1)cos(2pi.F.T)exp(-2pi.B.T)
%	See Slaney93, "An efficient implementation of the Patterson-Holdsworth
%	Auditory filter bank", para 3.0.
% dpwe 1994aug18  

if(exist('N')==0)
  N = 4;
end

G = T.^(N-1) .* cos(2*pi*F*T) .* exp(-2*pi*B*T);

