function X = ifilterbank(B,A,M,N)
% X = ifilterbank(B,A,M,N)      Invert IIR filterbank output to single channel.
%	Takes the matrix of parallel signals represented by M 
%	(each row is one channel), upsamples by a factor of N 
%	(i.e. inserts zeros if N>1) then filters each row by 
% 	the filter defined by the corresponding rows in A and B
%	before adding them all together to make an output signal, X.
% dpwe 1994jun21.  Uses built-in 'filter'

if(exist('N')==0)
  N = 1;
end

% recover number of filters
s = size(A);
bands = s(1);  % number of rows in A
s = size(M);
if s(1) ~= bands
  error('Rows of data different from number of filter definitions')
end

% find columns in M
mcols = s(2);
% columns in output may be interpolated
xcols = 1+(mcols-1)*N;

% initialize output
X = zeros(1,xcols);

% calculate each row
for filt = 1:bands
  % get filter parameters
  a = A(filt, :);
  b = B(filt, :);
  % extract data row & interpolate with zeros
  interp = zeros(1,xcols);
  interp(1:N:xcols) = M(filt,:);
  % filter
  y = filter(b,a,interp);
  % accumulate in output
  X = X+y;
end

