function [B,A,T,BW]=iirbpfilt(SR,CF,Q,N,MODE)
% [B,A,T,BW] = iirbpfilt(SR,CF,Q,N,MODE)   Design a single IIR bandpass filter.
%	Designs a recursive bandpass filter with center-frequency <CF>
%	(relative to a sampling rate <SR>), quality factor <Q> 
%	(i.e. CF/(-3dB bandwidth) == Q) and order 2*N (order must 
%	be even, so N = 1,2,3...).
%	MODE == 1 puts a zero at d.c. 2 puts one at Fny
%	T is a vector of group delays in samples for each filter.
% 	BW is the bandwidth in Hz.
% dpwe 1994jun20  after src/filtr/cqgram.c.  Uses built-in 'conv'

% Basic idea is that for a pole close to the unit circle, the 
% center frequency is given by theta/2pi*SR and the 3 dB bandwidth is 
% 2*(1-r).  Thus (1-r) = (CF/Q)/2  => r = 1 - (CF/2Q)

% if(exist('MODE')==0)
if(nargin < 5)
  MODE = 0;
end

designQ = Q/N;
omega = 2*pi/SR*CF;
bandwidth = omega/designQ;
r = 1-(bandwidth/2);
theta = omega;

% basic 2nd order filter: (z-r.exp(j.theta))(z-r.exp(-j.theta))
%  = z^2 + z(-2.r.cos(theta)) + r^2
AA = [1 (-2*r*cos(theta)) r*r];

% just cascade this N times

A = [1];

for x =1:N
  A = conv(A, AA);
end

% normalize the gain ..?
  B = (omega*omega*(1-r)^N);
if rem(MODE,2) == 1
  B = conv([1 -1],B);  % single zero at D.C.
  A = conv([1 -0.99],A); % add a pole nearby
end
if fix(MODE/2) == 1
  B = conv([.5 .5],B);  % single zero at Fnyq
end

T = SR*(N-1)./((4*pi/Q)*CF);
% T = talign(B,A,N);
BW = CF/Q;
