function [X,U] = invsurf(M,SR,SSR,B,A,T,W,BB)
% [X,U] = invsurf(M,SSR,SR,B,A,T,W,BB)  Invert energy envelope into sound.
%       M is an array of spectral envelope data sampled at SSR
%       SR is the desired sampling rate of the output sound.
%	B,A,T,W define a filterbank as well as its time delay and bandwidths,
%	BB is a matrix of filter inner products, calculated with iirbasis.m
%	if not provided.
%	See invweftinit.m to set up SR, B, A, T, W, BB for quick bulk proc.
%	U returns the magnitude surface actually used after nnls inversion.
% dpwe@media.mit.edu 1996feb02

% [M,SSR]=DATRead(F);
BANDS=size(M,1);

if(nargin<8)
  BB = iirbasis(B,A,4096);   % 4096 pt spectra for calculation
end

um = zeros(size(M));
disp('deconvolving frequency responses...');
% Make the deconvolved frequency envelope
for c=1:size(M,2)
  % Divide targets by bandwidths to normalize that energy factor
%  um(:,c) = nnls(BB,M(:,c)./sqrt(W));
  um(:,c) = nnls(BB,M(:,c).*sqrt(W));
  % Maybe this is already factored into .. the heights of BB???
  % Seems like BB takes out a factor of W (i.e. ...)
  % Anyway, this factor does about the right thing
end

% Construct noise excitation
ex = randn(1,SR/SSR*size(M,2));

% Apply the surface
disp('applying filter surface...');
LINEUP = 0.2;    % apparently need this constant to make amps match orig
X = LINEUP*surffilt(ex,um,SR/SSR,B,A,T);
% X = LINEUP*surffiltT(ex,um',SR/SSR,B,A,T);

U = um;
