function [X,Y,IR] = plotiir(B,A,SR,N,FM,C,P,T)
% [X,Y,IR] = plotiir(B,A,SR,N,FM,C,P) Plot frequency response of an IIR filter.
%	Plot the filter response of an IIR filter specified by B and A.
%	Use an N-point fft (defaults to 1024) and treat the maximum 
%	frequency as SR (defaults to 2pi). The spectrum is only 
%	displayed up to frequency FM (defaults to SR/2)
%	C is optional color specifier (default 'yr' ('r' is for phase)).
% 	Optional return codes may be used to recreate the graph with 
%	plot(X,Y);  IR returns the time-domain impulse response.
%	If P is specified nonzero, plot the phase too.
%	If T is specified, offset this amount of pure delay from the phase.
% dpwe 1994jun21.  Uses dB.m and built-ins 'filter', 'fft', 'plot'

% handle optional args
if(nargin < 3)
  SR = 2*pi;
end
if(nargin < 4)
  N = 1024;
end
if(nargin < 5)
  FM = SR/2;
end
if(nargin < 6)
  C = 'yr';
end
if(nargin < 7)
  P = 0;
end
if(nargin < 8)
  T = 0;
end

impulse = zeros(1, N);
%% impulse(1+floor(N/2)) = 1;
impulse(1) = 1;

ir = filter(B,A,impulse);
spec = fft(ir, N);

ff = (0:(N-1))*SR/N;
M  = FM/SR*N;
xx = 1:M;
X = ff(xx);
Y = dB(spec(xx));
PH = unwrap(angle(spec(xx))) + T/SR*2*pi*ff(xx);
if(PH(2) > 0)
  PH = PH - 2*pi*(1+floor(PH(2)/(2*pi)));
end

if(P>0)
  subplot(211)
  plot(X,Y,C(1));
  subplot(212)
  plot(X,PH,C(2));
else
  plot(X,Y,C(1));
end
IR = ir;
