function plotiirbank(B,A,SR,N,FM,P,T)
% plotiirbank(B,A,SR,N,M,P) Plot frequency responses of a whole IIR filterbank.
% 	Takes B and A as matrices where each row is the coefficients of 
%	one IIR filter.  Plots the spectra of all the filters on a single 
% 	graph.  SR (sample rate) defaults to 2pi.  N (spectrum order) 
%	defaults to 1024.  FM (highest freq displayed) defaults to SR/2.
%	P, if nonzero, causes phase to be plotted too.
%	T, if specified, is a time-delay subtracted from each phase curve.
% dpwe 1994jun21.  Uses plotiir.m

% handle default args (cannot pass undefined args through to plotiir?)
if(nargin < 3)
  SR = 2*pi;
end
if(nargin < 4)
  N = 1024;
end
if(nargin < 5)
  FM = SR/2;
end
if(nargin < 6)
  P=0;
end
if(nargin < 7)
  T=zeros(1,size(B,1));
end

C='yr';

% recover size of input array
s = size(A);
bands = s(1);

% first plot is special
plotiir(B(1,:), A(1,:), SR, N, FM, C, P, T(1));
% now cause others to be overplotted
if(P>0)
  subplot(211);
  hold;
  subplot(212);
  hold;
else
  hold;
end
for filter = 2:bands
  plotiir(B(filter,:), A(filter,:), SR, N, FM, C, P, T(filter));
end
% release the plot again
if(P>0)
  subplot(211);
  hold;
  subplot(212);
  hold;
else
  hold;
end


