function D=sgram(X,F,W,H,M,SL)
% D=sgram(X,F,W,H,M,SL)                                Plot a spectrogram of X.
%	Use an F-point fft (defaults to 1024), use a hanning window of 
%	W points (default: F) and hop the signal by H samples (default: W/2)
%	between each column.  Show just the lowest M bins (default: F/2, max).
%	imgsc(D) reproduces the image.
%	If SL is present & nonzero, emulates the Slaney/Lyon spectrogram 
%	by using sqrt(mag()) scaling and 3x3 blurring.  Else just unsmoothed
%	dB, 50 dB white-black range, from max.
% dpwe 1994jun24.  Uses stft.m, imgsc.m

% handle optional args
if(nargin < 2)
  F = 1024;
end
if(nargin < 3)
  W = F;
end
if(nargin < 4)
  H = W/2;
end
if(nargin < 5)
  M = F/2;
end
if(nargin < 6)
  SL = 0;
end

if(SL>0)
  % pre-emphasis hack copied from malcolm/dick's version
  X = filter([1 -0.95],1,X);
end

spec = stft(X, F, W, H);

if (SL>0)
  % full slaney-ism : scale as sqrt of ampl (4th root of power), 
  % blur by a 3x3 to make it look better

  kern = [.35 .5 .35;.5 1 .5;.35 .5 .35];  %  approx 1/2r for 9 pixels
  A = conv2(sqrt(abs(spec)), kern, 'same');
  d = A(1:M,:);
else
  mm = (spec==0);
  smin = minn(spec+maxx(spec)*mm);
  spec = spec+smin*mm;
  dbspec = 8.686*log(abs(spec(1:M,:)));
  % Scale plot to have a 50 dB range from black to white
  % (i.e. 'some threshold hack')
  mx = max(max(dbspec));
  d=(mfloor(dbspec, mx-50.0));
end

imgsc(d);

if(exist('D'))
  D = d;
end

