function Y = subsmooth(X,N,W)
% Y = subsmooth(X,N,W)                 Smooth and subsample rows of a matrix.
%	Elements of X are smoothed over a hanning window of length W 
%	(default 2*N+1), then subsampled by a factor of N.  The zero'th 
%	element is centered over the zero'th element of the input.
%	If W is a vector and X is a vector, each row is smoothed by a window 
%	of the corresponding width.  N, however, can only be a scalar.
%	(was As subsmooth, but uses conv to be faster for N=1)
%	SCALING is so that peak values remain about the same (conserve ampl).
%       1996jan17: filter applied to signal padded out with end vals.
% dpwe 1994jun22, 1994nov13

if(nargin < 3)
  W = 2*N+1;
end

sz = size(X);
sx = sz(2);
rows = sz(1);

% allocate output array
Y = zeros(rows, floor((sx+N-1)/N));

nw = prod(size(W));

if nw == 1   % all rows have same smooth factor
  % make weighting window
  sw = W(1);	% W only has one element anyway
%  w = hanning(sw)';
%  hanning relies on sigprocbox
  w = .5*(1 - cos(2*pi*(1:sw)/(sw+1)))/(0.5*(sw+1));   % last divisr is scaling

  % loop to calculate result
  for r = 1:rows
    ppp = X(r,1)*ones(1,sw-1);
    qqq = X(r,sx)*ones(1,sw-1);
    I = conv(w,[ppp,X(r,:),qqq]);
    Y(r, :) = I(floor(sw/2)+(sw-1)+[1:N:sx]);
  end
elseif nw == rows
  % different W factor for each row - smooth them individually
  for r = 1:rows
    sw = W(r);
    % make weighting window
    w = hanning(sw)';
    w = w./sum(w);	% give it unity sum
    ppp = X(r,1)*ones(1,sw-1);
    qqq = X(r,sx)*ones(1,sw-1);
    I = conv(w,[ppp,X(r,:),qqq]);
    Y(r, :) = I(floor(sw/2)+(sw-1)+[1:N:sx]);
  end
else
  error('window size must be a scalar, or have one element for each data row')
end

%% function Y = subsmooth(X,N,W)
%% % Y = subsmooth(X,N,W)                   Smooth and subsample rows of a matrix.
%% %	Elements of X are smoothed over a hanning window of length W 
%% %	(default 2*N+1), then subsampled by a factor of N.  The zero'th 
%% %	element is centered over the zero'th element of the input.
%% %	If W is a vector and X is a vector, each row is smoothed by a window 
%% %	of the corresponding width.  N, however, can only be a scalar.
%% % dpwe 1994jun22
%% 
%% % if (exist('W')== 0) 
%% %  W = 2*N+1;
%% % end
%% 
%% sz = size(X);
%% sx = sz(2);
%% rows = sz(1);
%% 
%% % allocate output array
%% Y = zeros(rows, floor(sx/N));
%% 
%% sw = prod(size(W));
%% 
%% if sw == 1   % all rows have same smooth factor
%%   % pad X with zeros to allow runoff
%%   X = [zeros(rows,floor(W/2)), X, zeros(rows,W-floor(W/2))];
%% 
%%   % make weighting window
%%   w = hanning(W)';
%%   w = w./sum(w);	% give it unity sum
%% 
%%   % loop to calculate result
%%   for step = 0:(floor(sx/N)-1)
%%     d = X(:,step*N+[1:W]);
%%     Y(:,1+step) = d*w';
%%   end
%% elseif sw == rows
%%   % different W factor for each row - smooth them individually
%%   for r = 1:rows
%%     % pad X with zeros to allow runoff
%%     ww = W(r);
%%     xx = [zeros(1,floor(ww/2)), X(r,:), zeros(1,ww-floor(ww/2))];
%% 
%%     % make weighting window
%%     w = hanning(ww)';
%%     w = w./sum(w);	% give it unity sum
%% 
%%     % loop to calculate result
%%     for step = 0:(floor(sx/N)-1)
%%       d = xx(step*N+[1:ww]);
%%       Y(r,1+step) = d*w';
%%     end
%%   end
%% else
%%   error('window size must be a scalar, or have one element for each data row')
%% end
%% 