function Y = surffilt(X,M,N,B,A,T,UF)
% Y = surffilt(X,M,N,B,A,T,UF)  Filter a signal by an interpolated t-f surface.
%	X is a sampled signal, and Y is the filtered version of it resulting 
%	from (1) passing X through the filterbank constituted by B,A
%	     (2) multiplying the each channel by the corresponding row 
%	         of M, interpolated by factor N, padded to right size
%	     (3) refiltering, combining results.
%	If X is a scalar rather than a vector, use unit normal white noise 
%	as excitation, whose length is the scalar X
%	i.e. approximately what ChanSource -s 4 is meant to do.
%	T is a optional set of time-alignments to apply to each channel, 
%	as returned by bpfiltbank
%	If UF is present and nonzero, use fracdelay to align the times, 
%	not simple integer delay
% dpwe 1995jan22

if(nargin < 6)	T = zeros(1, size(M,2)); end;
if(nargin < 7)	UF= 0; end;
Tmax = maxx(T);

disp('my surffilt');

nr = size(M,1);
if(vsize(X)==1)
  X = randn(nr,X);
end

%% fx = filterbank(B,A,X,1);
%% don't prefilter, just reproduce input signal on each row
%fx = ones(size(M,1),1)*X;

%[fr,fc] = size(fx);

%im = slinterp(M,N);

%Y = ifilterbank(B,A,fx.*resize(im, fr, fc));

% do it all by row to save core

if (nr ~= size(B,1))
  error('Rows in matrix and filter defs must match')
end

% do we pad or truncate to make cols(M)*N == cols(X)?
[xrows,xcols] = size(X);
%pad = xcols - (N*(size(M,2)-1)+1)
pad = xcols - (N*(size(M,2)));

Y = zeros(1, xcols+1);

for r = 1:nr
  disp(['row ' int2str(r) '...']);
  % for each row, interpolate the row of M, multiply it by the excitation, 
  % post-filter, add to output
  mrow = slinterp([M(r,:),0], N);
  if(pad > 0)
    mrow = [mrow, zeros(1, pad)];
  end 
  if(pad < 0)
    mrow = mrow(1:(xcols+1));
  end
  xrow = [X(1+rem((r-1), xrows),:),0];
  % delay X to compensate for filter delay
  if(UF==0)
    rdelay = round(Tmax - T(r));
    if(rdelay~=0)
      xrow = [zeros(1,rdelay), xrow(1:(vsize(xrow)-rdelay))];
    end
  else
    rdelay = Tmax-T(r);
    xrow = fracdelay(xrow,rdelay);  
  end
  Y = Y + filter(B(r,:), A(r,:), mrow.*xrow);
end
Y = Y(1:xcols);
