function T = talign(B,A,N,M)
% T = talign(B,A,N,M)        Find time-alignments for impulse-response extrema.
%	Find the (sample) time alignments for the impulse responses of 
%	each of a bank of (IIR) filters.  An impulse is filtered 
%	by each row of [B,A] and the sample time of the n'th extrema 
%	is returned (or zero if no maximum is found).  Maxima 
%	are located in the first m samples of response.  N defaults to 1, 
%	M to 1024.
% dpwe 1994jun27. Uses filterbank.m, localmaxfilt.m, rot.m, builtin 'cumsum'

if(exist('N')==0)
  N = 1;
end
if(exist('M')==0)
  M = 1024;
end

sz = size(B);
nfilts = sz(1);

impls = zeros(1,M);
impls(1) = 1;

% apply the filters
fimp = filterbank(B,A,impls);

% square so that maxima and minima become maxima
fimp = fimp.*fimp;

% find the maxima
fmx = localmaxfilt(fimp);

% count the maxima
fmc = cumsum(fmx')';

% find the numbered maxima
fmm = fmx .* (fmc == N);

% figure out the bin number of the remaining spikes
tt = ones(nfilts,1)*[1:M];
T = sum((tt.*fmm)');

