function [B,A,T,BW]=twoptwozfilt(SR,CF,Q,N)
% [B,A,T,BW] = twoptwozfilt(SR,CF,Q)    Design single bp filter, 2pole,z.
%	Designs a bandpass filter with center-frequency <CF>
%	(relative to a sampling rate <SR>), quality factor <Q> 
%	(i.e. CF/(-3dB bandwidth) == Q) and order 2.
%	This filter has one pair of complex poles, and zeros at dc and fnyq
%	If N is present and greater than 1, the poles are duplicated N times.
%	T is a vector of group delays in samples for each filter.
% 	BW is the bandwidth in Hz.
% dpwe 1995jan20 after iirbpfilt.m

if(nargin < 4)
  N = 1;
end

cfrad = 2*pi*CF/SR;
[r,theta] = twoptwozparams(SR,CF,Q,N);

AA = [1 -2*r*cos(theta) r*r];
A = [1];
for n = 1:N
  A = conv(A,AA);
end

% figure magnitude at centre freq & normalize it

p1 = r * exp(j*theta);
p2 = conj(p1);
z  = exp(j*cfrad);

mag = abs((1-z)*(-1-z))/(abs((p1-z)*(p2-z))^N);

B = (1/mag)*[1 0 -1];

BW = CF/Q;
% T  = Q*SR/CF;	% group delay in samples

% for N = 1, envelope is max at zero
% for N = 2, IR envelope is n r^n, so max when n.log(r) = -1
% for N = 3, IR env is n.n.r^n, max when n*n*lnr = -2
T = -(N-1)/log(r);
