function s = wfft(sig, winlen, fftlen)
% S = wfft(X, W, L)                            Windowed fast-Fourier transform.
%	Windows the data X with a centered Hanning window of length W,
%	then performs an L point fft on the result.
%	If L or W are omitted, they default to the length of X.
% dpwe 1994may23.  Uses: vsize.m, rot.m

siglen = vsize(sig);

% force sig to be a flat row
sig = sig(1:siglen);
s = size(sig);
if s(1) > 1
  sig = sig';
end

if exist('fftlen') == 0
  fftlen = siglen;
end

if exist('winlen') == 0
  winlen = siglen;
end

% ensure windowlength is odd.  If even, add one
if rem(winlen,2) == 0
  winlen = winlen + 1;
end

% pad or truncate signal to fftlen
sig = rot([sig,zeros(1,fftlen)], round((siglen-fftlen)/2));
sig = sig(1:fftlen);

% make window
win = 0.5*(1-cos([0:(winlen-1)]*2*pi/(winlen-1)));
% pad/trun to fftlen as for signal
win = rot([win,zeros(1,fftlen)], round(-.1+((winlen)-fftlen)/2));
win = win(1:fftlen);

d = win.*sig;
shlen = round( fftlen/2 );
d = rot(d, shlen);
s = fft(d);
% s = d;
